'''OpenGL extension EXT.texture_snorm

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_snorm to provide a more 
Python-friendly API

Overview (from the spec)
	
	Fixed-point textures in unextended OpenGL have integer components,
	but those values are taken to represent floating-point values in
	the range [0.0,1.0]. These integer components are considered
	"unsigned normalized" integers. When such a texture is accessed by
	a shader or by fixed-function fragment processing, floating-point
	values are returned in the range [0.0,1.0].
	
	This extension provides a set of new "signed normalized" integer
	texture formats. These are taken to represent a floating-point
	value in the range [-1.0,1.0] with an exact 0.0.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_snorm.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.texture_snorm import *
from OpenGL.raw.GL.EXT.texture_snorm import _EXTENSION_NAME

def glInitTextureSnormEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.shader_atomic_counter_ops

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.shader_atomic_counter_ops to provide a more 
Python-friendly API

Overview (from the spec)
	
	The ARB_shader_atomic_counters extension introduced atomic counters, but
	it limits list of potential operations that can be performed on them to
	increment, decrement, and query. This extension extends the list of GLSL
	built-in functions that can operate on atomic counters. The list of new
	operations include:
	
	  * Increment and decrement with wrap
	  * Addition and subtraction
	  * Minimum and maximum
	  * Bitwise operators (AND, OR, XOR, etc.)
	  * Masked OR operator
	  * Exchange, and compare and exchange operators

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/shader_atomic_counter_ops.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.shader_atomic_counter_ops import *
from OpenGL.raw.GL.AMD.shader_atomic_counter_ops import _EXTENSION_NAME

def glInitShaderAtomicCounterOpsAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ATI_envmap_bumpmap'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ATI_envmap_bumpmap',error_checker=_errors._error_checker)
GL_BUMP_ENVMAP_ATI=_C('GL_BUMP_ENVMAP_ATI',0x877B)
GL_BUMP_NUM_TEX_UNITS_ATI=_C('GL_BUMP_NUM_TEX_UNITS_ATI',0x8777)
GL_BUMP_ROT_MATRIX_ATI=_C('GL_BUMP_ROT_MATRIX_ATI',0x8775)
GL_BUMP_ROT_MATRIX_SIZE_ATI=_C('GL_BUMP_ROT_MATRIX_SIZE_ATI',0x8776)
GL_BUMP_TARGET_ATI=_C('GL_BUMP_TARGET_ATI',0x877C)
GL_BUMP_TEX_UNITS_ATI=_C('GL_BUMP_TEX_UNITS_ATI',0x8778)
GL_DU8DV8_ATI=_C('GL_DU8DV8_ATI',0x877A)
GL_DUDV_ATI=_C('GL_DUDV_ATI',0x8779)
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glGetTexBumpParameterfvATI(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glGetTexBumpParameterivATI(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glTexBumpParameterfvATI(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glTexBumpParameterivATI(pname,param):pass



'''OpenGL extension OES.texture_cube_map

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.texture_cube_map to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_cube_map.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.texture_cube_map import *
from OpenGL.raw.GLES1.OES.texture_cube_map import _EXTENSION_NAME

def glInitTextureCubeMapOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glTexGenfvOES.params size not checked against 'pname'
glTexGenfvOES=wrapper.wrapper(glTexGenfvOES).setInputArraySize(
    'params', None
)
# INPUT glTexGenivOES.params size not checked against 'pname'
glTexGenivOES=wrapper.wrapper(glTexGenivOES).setInputArraySize(
    'params', None
)
# INPUT glTexGenxvOES.params size not checked against 'pname'
glTexGenxvOES=wrapper.wrapper(glTexGenxvOES).setInputArraySize(
    'params', None
)
# INPUT glGetTexGenfvOES.params size not checked against 'pname'
glGetTexGenfvOES=wrapper.wrapper(glGetTexGenfvOES).setInputArraySize(
    'params', None
)
# INPUT glGetTexGenivOES.params size not checked against 'pname'
glGetTexGenivOES=wrapper.wrapper(glGetTexGenivOES).setInputArraySize(
    'params', None
)
glGetTexGenxvOES=wrapper.wrapper(glGetTexGenxvOES).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GL_1_1

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_1_1 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_1_1.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_1_1 import *
from OpenGL.raw.GL.VERSION.GL_1_1 import _EXTENSION_NAME

def glInitGl11VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawElements.indices size not checked against 'count,type'
glDrawElements=wrapper.wrapper(glDrawElements).setInputArraySize(
    'indices', None
)
glGetPointerv=wrapper.wrapper(glGetPointerv).setOutput(
    'params',size=(1,),orPassIn=True
)
# INPUT glTexSubImage1D.pixels size not checked against 'format,type,width'
glTexSubImage1D=wrapper.wrapper(glTexSubImage1D).setInputArraySize(
    'pixels', None
)
# INPUT glTexSubImage2D.pixels size not checked against 'format,type,width,height'
glTexSubImage2D=wrapper.wrapper(glTexSubImage2D).setInputArraySize(
    'pixels', None
)
# INPUT glDeleteTextures.textures size not checked against n
glDeleteTextures=wrapper.wrapper(glDeleteTextures).setInputArraySize(
    'textures', None
)
glGenTextures=wrapper.wrapper(glGenTextures).setOutput(
    'textures',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
# INPUT glColorPointer.pointer size not checked against 'size,type,stride'
glColorPointer=wrapper.wrapper(glColorPointer).setInputArraySize(
    'pointer', None
)
# INPUT glEdgeFlagPointer.pointer size not checked against 'stride'
glEdgeFlagPointer=wrapper.wrapper(glEdgeFlagPointer).setInputArraySize(
    'pointer', None
)
# INPUT glIndexPointer.pointer size not checked against 'type,stride'
glIndexPointer=wrapper.wrapper(glIndexPointer).setInputArraySize(
    'pointer', None
)
# INPUT glInterleavedArrays.pointer size not checked against 'format,stride'
glInterleavedArrays=wrapper.wrapper(glInterleavedArrays).setInputArraySize(
    'pointer', None
)
# INPUT glNormalPointer.pointer size not checked against 'type,stride'
glNormalPointer=wrapper.wrapper(glNormalPointer).setInputArraySize(
    'pointer', None
)
# INPUT glTexCoordPointer.pointer size not checked against 'size,type,stride'
glTexCoordPointer=wrapper.wrapper(glTexCoordPointer).setInputArraySize(
    'pointer', None
)
# INPUT glVertexPointer.pointer size not checked against 'size,type,stride'
glVertexPointer=wrapper.wrapper(glVertexPointer).setInputArraySize(
    'pointer', None
)
# INPUT glAreTexturesResident.textures size not checked against n
glAreTexturesResident=wrapper.wrapper(glAreTexturesResident).setOutput(
    'residences',size=lambda x:(x,),pnameArg='n',orPassIn=True
).setInputArraySize(
    'textures', None
)
# INPUT glPrioritizeTextures.priorities size not checked against n
# INPUT glPrioritizeTextures.textures size not checked against n
glPrioritizeTextures=wrapper.wrapper(glPrioritizeTextures).setInputArraySize(
    'priorities', None
).setInputArraySize(
    'textures', None
)
glIndexubv=wrapper.wrapper(glIndexubv).setInputArraySize(
    'c', 1
)
### END AUTOGENERATED SECTION
from OpenGL.GL.VERSION.GL_1_0 import *
GL_MODELVIEW0_EXT = GL_MODELVIEW # alias
GL_MODELVIEW0_MATRIX_EXT = GL_MODELVIEW_MATRIX # alias
GL_MODELVIEW0_STACK_DEPTH_EXT = GL_MODELVIEW_STACK_DEPTH # alias
GL_TEXTURE_COMPONENTS = GL_TEXTURE_INTERNAL_FORMAT # alias

# See: http://lists.freedesktop.org/archives/mesa-dev/2011-October/013095.html
# basically these names were present for a while in the .spec files but were not 
# actually used officially, the mapped values here are what is supposed to get 
# used instead.
GL_DEPTH_BUFFER = GL_DEPTH
GL_STENCIL_BUFFER = GL_STENCIL



'''OpenGL extension NV.fragment_program4

This module customises the behaviour of the 
OpenGL.raw.GL.NV.fragment_program4 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds on the common assembly instruction set
	infrastructure provided by NV_gpu_program4, adding fragment
	program-specific features.
	
	This extension provides interpolation modifiers to fragment program
	attributes allowing programs to specify that specified attributes be
	flat-shaded (constant over a primitive), centroid-sampled (multisample
	rendering), or interpolated linearly in screen space.  The set of input
	and output bindings provided includes all bindings supported by
	ARB_fragment_program.  Additional input bindings are provided to determine
	whether fragments were generated by front- or back-facing primitives
	("fragment.facing"), to identify the individual primitive used to generate
	the fragment ("primitive.id"), and to determine distances to user clip
	planes ("fragment.clip[n]").  Additionally generic input attributes allow
	a fragment program to receive a greater number of attributes from previous
	pipeline stages than possible using only the pre-defined fixed-function
	attributes.
	
	By and large, programs written to ARB_fragment_program can be ported
	directly by simply changing the program header from "!!ARBfp1.0" to
	"!!NVfp4.0", and then modifying instructions to take advantage of the
	expanded feature set.  There are a small number of areas where this
	extension is not a functional superset of previous fragment program
	extensions, which are documented in the NV_gpu_program4 specification.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/fragment_program4.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.fragment_program4 import *
from OpenGL.raw.GL.NV.fragment_program4 import _EXTENSION_NAME

def glInitFragmentProgram4NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_OES_byte_coordinates'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_OES_byte_coordinates',error_checker=_errors._error_checker)
GL_BYTE=_C('GL_BYTE',0x1400)
@_f
@_p.types(None,_cs.GLenum,_cs.GLbyte)
def glMultiTexCoord1bOES(texture,s):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLbyteArray)
def glMultiTexCoord1bvOES(texture,coords):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLbyte,_cs.GLbyte)
def glMultiTexCoord2bOES(texture,s,t):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLbyteArray)
def glMultiTexCoord2bvOES(texture,coords):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glMultiTexCoord3bOES(texture,s,t,r):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLbyteArray)
def glMultiTexCoord3bvOES(texture,coords):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glMultiTexCoord4bOES(texture,s,t,r,q):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLbyteArray)
def glMultiTexCoord4bvOES(texture,coords):pass
@_f
@_p.types(None,_cs.GLbyte)
def glTexCoord1bOES(s):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glTexCoord1bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte)
def glTexCoord2bOES(s,t):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glTexCoord2bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glTexCoord3bOES(s,t,r):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glTexCoord3bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glTexCoord4bOES(s,t,r,q):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glTexCoord4bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte)
def glVertex2bOES(x,y):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glVertex2bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glVertex3bOES(x,y,z):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glVertex3bvOES(coords):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glVertex4bOES(x,y,z,w):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glVertex4bvOES(coords):pass



'''OpenGL extension EXT.shadow_funcs

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.shadow_funcs to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension generalizes the GL_ARB_shadow extension to support all
	eight binary texture comparison functions rather than just GL_LEQUAL
	and GL_GEQUAL.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shadow_funcs.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.shadow_funcs import *
from OpenGL.raw.GL.EXT.shadow_funcs import _EXTENSION_NAME

def glInitShadowFuncsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_EXT_texture_from_pixmap'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_EXT_texture_from_pixmap',error_checker=_errors._error_checker)
GLX_AUX0_EXT=_C('GLX_AUX0_EXT',0x20E2)
GLX_AUX1_EXT=_C('GLX_AUX1_EXT',0x20E3)
GLX_AUX2_EXT=_C('GLX_AUX2_EXT',0x20E4)
GLX_AUX3_EXT=_C('GLX_AUX3_EXT',0x20E5)
GLX_AUX4_EXT=_C('GLX_AUX4_EXT',0x20E6)
GLX_AUX5_EXT=_C('GLX_AUX5_EXT',0x20E7)
GLX_AUX6_EXT=_C('GLX_AUX6_EXT',0x20E8)
GLX_AUX7_EXT=_C('GLX_AUX7_EXT',0x20E9)
GLX_AUX8_EXT=_C('GLX_AUX8_EXT',0x20EA)
GLX_AUX9_EXT=_C('GLX_AUX9_EXT',0x20EB)
GLX_BACK_EXT=_C('GLX_BACK_EXT',0x20E0)
GLX_BACK_LEFT_EXT=_C('GLX_BACK_LEFT_EXT',0x20E0)
GLX_BACK_RIGHT_EXT=_C('GLX_BACK_RIGHT_EXT',0x20E1)
GLX_BIND_TO_MIPMAP_TEXTURE_EXT=_C('GLX_BIND_TO_MIPMAP_TEXTURE_EXT',0x20D2)
GLX_BIND_TO_TEXTURE_RGBA_EXT=_C('GLX_BIND_TO_TEXTURE_RGBA_EXT',0x20D1)
GLX_BIND_TO_TEXTURE_RGB_EXT=_C('GLX_BIND_TO_TEXTURE_RGB_EXT',0x20D0)
GLX_BIND_TO_TEXTURE_TARGETS_EXT=_C('GLX_BIND_TO_TEXTURE_TARGETS_EXT',0x20D3)
GLX_FRONT_EXT=_C('GLX_FRONT_EXT',0x20DE)
GLX_FRONT_LEFT_EXT=_C('GLX_FRONT_LEFT_EXT',0x20DE)
GLX_FRONT_RIGHT_EXT=_C('GLX_FRONT_RIGHT_EXT',0x20DF)
GLX_MIPMAP_TEXTURE_EXT=_C('GLX_MIPMAP_TEXTURE_EXT',0x20D7)
GLX_TEXTURE_1D_BIT_EXT=_C('GLX_TEXTURE_1D_BIT_EXT',0x00000001)
GLX_TEXTURE_1D_EXT=_C('GLX_TEXTURE_1D_EXT',0x20DB)
GLX_TEXTURE_2D_BIT_EXT=_C('GLX_TEXTURE_2D_BIT_EXT',0x00000002)
GLX_TEXTURE_2D_EXT=_C('GLX_TEXTURE_2D_EXT',0x20DC)
GLX_TEXTURE_FORMAT_EXT=_C('GLX_TEXTURE_FORMAT_EXT',0x20D5)
GLX_TEXTURE_FORMAT_NONE_EXT=_C('GLX_TEXTURE_FORMAT_NONE_EXT',0x20D8)
GLX_TEXTURE_FORMAT_RGBA_EXT=_C('GLX_TEXTURE_FORMAT_RGBA_EXT',0x20DA)
GLX_TEXTURE_FORMAT_RGB_EXT=_C('GLX_TEXTURE_FORMAT_RGB_EXT',0x20D9)
GLX_TEXTURE_RECTANGLE_BIT_EXT=_C('GLX_TEXTURE_RECTANGLE_BIT_EXT',0x00000004)
GLX_TEXTURE_RECTANGLE_EXT=_C('GLX_TEXTURE_RECTANGLE_EXT',0x20DD)
GLX_TEXTURE_TARGET_EXT=_C('GLX_TEXTURE_TARGET_EXT',0x20D6)
GLX_Y_INVERTED_EXT=_C('GLX_Y_INVERTED_EXT',0x20D4)
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXDrawable,_cs.c_int,ctypes.POINTER(_cs.c_int))
def glXBindTexImageEXT(dpy,drawable,buffer,attrib_list):pass
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXDrawable,_cs.c_int)
def glXReleaseTexImageEXT(dpy,drawable,buffer):pass



'''OpenGL extension SGIX.async_pixel

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.async_pixel to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a new asynchronous mode for texture
	download, pixel download and pixel readback commands.  It allows
	programs to transfer textures or images between the host and the
	graphics accelerator in parallel with the execution of other
	graphics commands (possibly taking advantage of a secondary path
	to the graphics accelerator).  It also allows programs to issue
	non-blocking pixel readback commands that return immediately after
	they are issued so that the program can issue other commands while
	the readback takes place.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/async_pixel.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.async_pixel import *
from OpenGL.raw.GL.SGIX.async_pixel import _EXTENSION_NAME

def glInitAsyncPixelSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Traditional PyOpenGL interface to Togl

This module provides access to the Tkinter Togl widget 
with a relatively "thick" wrapper API that creates a set 
of default "examination" operations.

Note that many (most) Linux distributions have 
now split out the Tkinter bindings into a separate package,
and that Togl must be separately downloaded (a script is 
provided in the source distribution which downloads and 
installs Togl 2.0 binaries for you).

Because of the complexity and fragility of the installation, 
it is recommended that you use Pygame, wxPython, PyGtk, or 
PyQt for real-world projects, and GLUT or Pygame for simple 
demo/testing interfaces.

The Togl project is located here:

    http://togl.sourceforge.net/
"""
# A class that creates an opengl widget.
# Mike Hartshorn
# Department of Chemistry
# University of York, UK
import os,sys, logging
_log = logging.getLogger( 'OpenGL.Tk' )
from OpenGL.GL import *
from OpenGL.GLU import *
try:
    from tkinter import _default_root
    from tkinter import *
    from tkinter import dialog
except ImportError as err:
    try:
        from Tkinter import _default_root
        from Tkinter import *
        import Dialog as dialog
    except ImportError as err:
        _log.error( """Unable to import Tkinter, likely need to install a separate package (python-tk) to have Tkinter support.  You likely also want to run the src/togl.py script in the PyOpenGL source distribution to install the Togl widget""" )
        raise
import math

def glTranslateScene(s, x, y, mousex, mousey):
    glMatrixMode(GL_MODELVIEW)
    mat = glGetDoublev(GL_MODELVIEW_MATRIX)
    glLoadIdentity()
    glTranslatef(s * (x - mousex), s * (mousey - y), 0.0)
    glMultMatrixd(mat)


def glRotateScene(s, xcenter, ycenter, zcenter, x, y, mousex, mousey):
    glMatrixMode(GL_MODELVIEW)
    mat = glGetDoublev(GL_MODELVIEW_MATRIX)
    glLoadIdentity()
    glTranslatef(xcenter, ycenter, zcenter)
    glRotatef(s * (y - mousey), 1., 0., 0.)
    glRotatef(s * (x - mousex), 0., 1., 0.)
    glTranslatef(-xcenter, -ycenter, -zcenter)
    glMultMatrixd(mat)


def sub(x, y):
    return list(map(lambda a, b: a-b, x, y))


def dot(x, y):
    t = 0
    for i in range(len(x)):
        t = t + x[i]*y[i]
    return t


def glDistFromLine(x, p1, p2):
    f = list(map(lambda x, y: x-y, p2, p1))
    g = list(map(lambda x, y: x-y, x, p1))
    return dot(g, g) - dot(f, g)**2/dot(f, f)


# Keith Junius <junius@chem.rug.nl> provided many changes to Togl
TOGL_NORMAL = 1
TOGL_OVERLAY = 2

def v3distsq(a,b):
    d = ( a[0] - b[0], a[1] - b[1], a[2] - b[2] )
    return d[0]*d[0] + d[1]*d[1] + d[2]*d[2]

# new version from Daniel Faken (Daniel_Faken@brown.edu) for static 
# loading comptability
if _default_root is None:
    _default_root = Tk()

# Add this file's directory to Tcl's search path
# This code is intended to work with the src/togl.py script 
# which will populate the directory with the appropriate 
# Binary Togl distribution.  Note that Togl 2.0 and above 
# uses "stubs", so we don't care what Tk version we are using,
# but that a different build is required for 64-bit Python.
# Thus the directory structure is *not* the same as the 
# original PyOpenGL versions.
import sys 
if sys.maxint > 2**32:
    suffix = '-64'
else:
    suffix = ''
try:
    TOGL_DLL_PATH = os.path.join(
        os.path.dirname(__file__),
        'togl-'+ sys.platform + suffix,
    )
except NameError as err:
    # no __file__, likely running as an egg
    TOGL_DLL_PATH = ""

if not os.path.isdir( TOGL_DLL_PATH ):
    _log.warning( 'Expected Tk Togl installation in %s', TOGL_DLL_PATH )
_log.info( 'Loading Togl from: %s', TOGL_DLL_PATH )
_default_root.tk.call('lappend', 'auto_path', TOGL_DLL_PATH)
_default_root.tk.call('package', 'require', 'Togl')
try:
    _default_root.tk.eval('load {} Togl')
except TclError as err:
    _log.error( """Failure loading Togl package: %s""", err )
    raise

# This code is needed to avoid faults on sys.exit()
# [DAA, Jan 1998], updated by mcfletch 2009
import atexit
def cleanup():
    try:
        import tkinter 
    except ImportError as err:
        import Tkinter as tkinter
    try: 
        if tkinter._default_root: tkinter._default_root.destroy()
    except (TclError,AttributeError) as err:
        pass
    tkinter._default_root = None
atexit.register( cleanup )

class Togl(Widget):
    """
    Togl Widget
    Keith Junius
    Department of Biophysical Chemistry
    University of Groningen, The Netherlands
    Very basic widget which provides access to Togl functions.
    """
    def __init__(self, master=None, cnf={}, **kw):
        Widget.__init__(self, master, 'togl', cnf, kw)


    def render(self):
        return


    def swapbuffers(self):
        self.tk.call(self._w, 'swapbuffers')


    def makecurrent(self):
        self.tk.call(self._w, 'makecurrent')


    def alloccolor(self, red, green, blue):
        return self.tk.getint(self.tk.call(self._w, 'alloccolor', red, green, blue))


    def freecolor(self, index):
        self.tk.call(self._w, 'freecolor', index)


    def setcolor(self, index, red, green, blue):
        self.tk.call(self._w, 'setcolor', index, red, green, blue)


    def loadbitmapfont(self, fontname):
        return self.tk.getint(self.tk.call(self._w, 'loadbitmapfont', fontname))


    def unloadbitmapfont(self, fontbase):
        self.tk.call(self._w, 'unloadbitmapfont', fontbase)


    def uselayer(self, layer):
        self.tk.call(self._w, 'uselayer', layer)


    def showoverlay(self):
        self.tk.call(self._w, 'showoverlay')


    def hideoverlay(self):
        self.tk.call(self._w, 'hideoverlay')


    def existsoverlay(self):
        return self.tk.getboolean(self.tk.call(self._w, 'existsoverlay'))


    def getoverlaytransparentvalue(self):
        return self.tk.getint(self.tk.call(self._w, 'getoverlaytransparentvalue'))


    def ismappedoverlay(self):
        return self.tk.getboolean(self.tk.call(self._w, 'ismappedoverlay'))


    def alloccoloroverlay(self, red, green, blue):
        return self.tk.getint(self.tk.call(self._w, 'alloccoloroverlay', red, green, blue))


    def freecoloroverlay(self, index):
        self.tk.call(self._w, 'freecoloroverlay', index)



class RawOpengl(Widget, Misc):
    """Widget without any sophisticated bindings\
    by Tom Schwaller"""


    def __init__(self, master=None, cnf={}, **kw):
        Widget.__init__(self, master, 'togl', cnf, kw)
        self.bind('<Map>', self.tkMap)
        self.bind('<Expose>', self.tkExpose)
        self.bind('<Configure>', self.tkExpose)


    def tkRedraw(self, *dummy):
        # This must be outside of a pushmatrix, since a resize event
        # will call redraw recursively. 
        self.update_idletasks()
        self.tk.call(self._w, 'makecurrent')
        _mode = glGetDoublev(GL_MATRIX_MODE)
        try:
            glMatrixMode(GL_PROJECTION)
            glPushMatrix()
            try:
                self.redraw()
                glFlush()
            finally:
                glPopMatrix()
        finally:
            glMatrixMode(_mode)
        self.tk.call(self._w, 'swapbuffers')


    def tkMap(self, *dummy):
        self.tkExpose()


    def tkExpose(self, *dummy):
        self.tkRedraw()




class Opengl(RawOpengl):
    """\
Tkinter bindings for an Opengl widget.
Mike Hartshorn
Department of Chemistry
University of York, UK
http://www.yorvic.york.ac.uk/~mjh/
"""

    def __init__(self, master=None, cnf={}, **kw):
        """\
        Create an opengl widget.
        Arrange for redraws when the window is exposed or when
        it changes size."""

        #Widget.__init__(self, master, 'togl', cnf, kw)
        RawOpengl.__init__(*(self, master, cnf), **kw)
        self.initialised = 0

        # Current coordinates of the mouse.
        self.xmouse = 0
        self.ymouse = 0

        # Where we are centering.
        self.xcenter = 0.0
        self.ycenter = 0.0
        self.zcenter = 0.0

        # The _back color
        self.r_back = 1.
        self.g_back = 0.
        self.b_back = 1.

        # Where the eye is
        self.distance = 10.0

        # Field of view in y direction
        self.fovy = 30.0

        # Position of clipping planes.
        self.near = 0.1
        self.far = 1000.0

        # Is the widget allowed to autospin?
        self.autospin_allowed = 0

        # Is the widget currently autospinning?
        self.autospin = 0

        # Basic bindings for the virtual trackball
        self.bind('<Map>', self.tkMap)
        self.bind('<Expose>', self.tkExpose)
        self.bind('<Configure>', self.tkExpose)
        self.bind('<Shift-Button-1>', self.tkHandlePick)
        #self.bind('<Button-1><ButtonRelease-1>', self.tkHandlePick)
        self.bind('<Button-1>', self.tkRecordMouse)
        self.bind('<B1-Motion>', self.tkTranslate)
        self.bind('<Button-2>', self.StartRotate)
        self.bind('<B2-Motion>', self.tkRotate)
        self.bind('<ButtonRelease-2>', self.tkAutoSpin)
        self.bind('<Button-3>', self.tkRecordMouse)
        self.bind('<B3-Motion>', self.tkScale)


    def help(self):
        """Help for the widget."""

        import Dialog
        d = Dialog.Dialog(None, {'title': 'Viewer help',
                                 'text': 'Button-1: Translate\n'
                                         'Button-2: Rotate\n'
                                         'Button-3: Zoom\n'
                                         'Reset: Resets transformation to identity\n',
                                 'bitmap': 'questhead',
                                 'default': 0,
                                 'strings': ('Done', 'Ok')})


    def activate(self):
        """Cause this Opengl widget to be the current destination for drawing."""

        self.tk.call(self._w, 'makecurrent')


    # This should almost certainly be part of some derived class.
    # But I have put it here for convenience.
    def basic_lighting(self):
        """\
        Set up some basic lighting (single infinite light source).

        Also switch on the depth buffer."""
   
        self.activate()
        light_position = (1, 1, 1, 0)
        glLightfv(GL_LIGHT0, GL_POSITION, light_position)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glDepthFunc(GL_LESS)
        glEnable(GL_DEPTH_TEST)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

    def set_background(self, r, g, b):
        """Change the background colour of the widget."""

        self.r_back = r
        self.g_back = g
        self.b_back = b

        self.tkRedraw()


    def set_centerpoint(self, x, y, z):
        """Set the new center point for the model.
        This is where we are looking."""

        self.xcenter = x
        self.ycenter = y
        self.zcenter = z

        self.tkRedraw()


    def set_eyepoint(self, distance):
        """Set how far the eye is from the position we are looking."""

        self.distance = distance
        self.tkRedraw()


    def reset(self):
        """Reset rotation matrix for this widget."""

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        self.tkRedraw()


    def tkHandlePick(self, event):
        """Handle a pick on the scene."""

        if hasattr(self, 'pick'):
            # here we need to use glu.UnProject

            # Tk and X have their origin top left, 
            # while Opengl has its origin bottom left.
            # So we need to subtract y from the window height to get
            # the proper pick position for Opengl

            realy = self.winfo_height() - event.y

            p1 = gluUnProject(event.x, realy, 0.)
            p2 = gluUnProject(event.x, realy, 1.)

            if self.pick(self, p1, p2):
                """If the pick method returns true we redraw the scene."""

                self.tkRedraw()


    def tkRecordMouse(self, event):
        """Record the current mouse position."""

        self.xmouse = event.x
        self.ymouse = event.y


    def StartRotate(self, event):
        # Switch off any autospinning if it was happening

        self.autospin = 0
        self.tkRecordMouse(event)


    def tkScale(self, event):
        """Scale the scene.  Achieved by moving the eye position.

        Dragging up zooms in, while dragging down zooms out
        """
        scale = 1 - 0.01 * (event.y - self.ymouse)
        # do some sanity checks, scale no more than
        # 1:1000 on any given click+drag
        if scale < 0.001:
            scale = 0.001
        elif scale > 1000:
            scale = 1000
        self.distance = self.distance * scale
        self.tkRedraw()
        self.tkRecordMouse(event)


    def do_AutoSpin(self):
        s = 0.5
        self.activate()

        glRotateScene(0.5, self.xcenter, self.ycenter, self.zcenter, self.yspin, self.xspin, 0, 0)
        self.tkRedraw()

        if self.autospin:
            self.after(10, self.do_AutoSpin)


    def tkAutoSpin(self, event):
        """Perform autospin of scene."""

        self.after(4)
        self.update_idletasks()

        # This could be done with one call to pointerxy but I'm not sure
        # it would any quicker as we would have to split up the resulting
        # string and then conv

        x = self.tk.getint(self.tk.call('winfo', 'pointerx', self._w))
        y = self.tk.getint(self.tk.call('winfo', 'pointery', self._w))

        if self.autospin_allowed:
            if x != event.x_root and y != event.y_root:
                self.autospin = 1

        self.yspin = x - event.x_root
        self.xspin = y - event.y_root

        self.after(10, self.do_AutoSpin)


    def tkRotate(self, event):
        """Perform rotation of scene."""

        self.activate()
        glRotateScene(0.5, self.xcenter, self.ycenter, self.zcenter, event.x, event.y, self.xmouse, self.ymouse)
        self.tkRedraw()
        self.tkRecordMouse(event)


    def tkTranslate(self, event):
        """Perform translation of scene."""

        self.activate()

        # Scale mouse translations to object viewplane so object tracks with mouse
        win_height = max( 1,self.winfo_height() )
        obj_c	  = ( self.xcenter, self.ycenter, self.zcenter )
        win		= gluProject( obj_c[0], obj_c[1], obj_c[2])
        obj		= gluUnProject( win[0], win[1] + 0.5 * win_height, win[2])
        dist	   = math.sqrt( v3distsq( obj, obj_c ) )
        scale	  = abs( dist / ( 0.5 * win_height ) )

        glTranslateScene(scale, event.x, event.y, self.xmouse, self.ymouse)
        self.tkRedraw()
        self.tkRecordMouse(event)


    def tkRedraw(self, *dummy):
        """Cause the opengl widget to redraw itself."""

        if not self.initialised: return
        self.activate()

        glPushMatrix()			# Protect our matrix
        self.update_idletasks()
        self.activate()
        w = self.winfo_width()
        h = self.winfo_height()
        glViewport(0, 0, w, h)

        # Clear the background and depth buffer.
        glClearColor(self.r_back, self.g_back, self.b_back, 0.)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(self.fovy, float(w)/float(h), self.near, self.far)

        if 0:
            # Now translate the scene origin away from the world origin
            glMatrixMode(GL_MODELVIEW)
            mat = glGetDoublev(GL_MODELVIEW_MATRIX)
            glLoadIdentity()
            glTranslatef(-self.xcenter, -self.ycenter, -(self.zcenter+self.distance))
            glMultMatrixd(mat)
        else:
            gluLookAt(self.xcenter, self.ycenter, self.zcenter + self.distance,
                self.xcenter, self.ycenter, self.zcenter,
                0., 1., 0.)
            glMatrixMode(GL_MODELVIEW)
    
        # Call objects redraw method.
        self.redraw(self)
        glFlush()				# Tidy up
        glPopMatrix()			# Restore the matrix

        self.tk.call(self._w, 'swapbuffers')
    def redraw( self, *args, **named ):
        """Prevent access errors if user doesn't set redraw fast enough"""


    def tkMap(self, *dummy):
        """Cause the opengl widget to redraw itself."""

        self.tkExpose()


    def tkExpose(self, *dummy):
        """Redraw the widget.
        Make it active, update tk events, call redraw procedure and
        swap the buffers.  Note: swapbuffers is clever enough to
        only swap double buffered visuals."""

        self.activate()
        if not self.initialised:
            self.basic_lighting()
            self.initialised = 1
        self.tkRedraw()


    def tkPrint(self, file):
        """Turn the current scene into PostScript via the feedback buffer."""

        self.activate()



'''OpenGL extension EXT.polygon_offset

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.polygon_offset to provide a more 
Python-friendly API

Overview (from the spec)
	
	The depth values of fragments generated by rendering polygons are
	displaced by an amount that is proportional to the maximum absolute
	value of the depth slope of the polygon, measured and applied in window
	coordinates.  This displacement allows lines (or points) and polygons
	in the same plane to be rendered without interaction -- the lines
	rendered either completely in front of or behind the polygons
	(depending on the sign of the offset factor).  It also allows multiple
	coplanar polygons to be rendered without interaction, if different
	offset factors are used for each polygon.  Applications include
	rendering hidden-line images, rendering solids with highlighted edges,
	and applying `decals' to surfaces.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/polygon_offset.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.polygon_offset import *
from OpenGL.raw.GL.EXT.polygon_offset import _EXTENSION_NAME

def glInitPolygonOffsetEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""The GLUT library implementation via ctypes"""
from OpenGL.raw.GLUT import *

from OpenGL.GLUT.special import *
from OpenGL.GLUT.fonts import *
from OpenGL.GLUT.freeglut import *
from OpenGL.GLUT.osx import *

if glutLeaveMainLoop:
    HAVE_FREEGLUT = True 
else:
    HAVE_FREEGLUT = False 



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_compute_variable_group_size'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_compute_variable_group_size',error_checker=_errors._error_checker)
GL_MAX_COMPUTE_FIXED_GROUP_INVOCATIONS_ARB=_C('GL_MAX_COMPUTE_FIXED_GROUP_INVOCATIONS_ARB',0x90EB)
GL_MAX_COMPUTE_FIXED_GROUP_SIZE_ARB=_C('GL_MAX_COMPUTE_FIXED_GROUP_SIZE_ARB',0x91BF)
GL_MAX_COMPUTE_VARIABLE_GROUP_INVOCATIONS_ARB=_C('GL_MAX_COMPUTE_VARIABLE_GROUP_INVOCATIONS_ARB',0x9344)
GL_MAX_COMPUTE_VARIABLE_GROUP_SIZE_ARB=_C('GL_MAX_COMPUTE_VARIABLE_GROUP_SIZE_ARB',0x9345)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glDispatchComputeGroupSizeARB(num_groups_x,num_groups_y,num_groups_z,group_size_x,group_size_y,group_size_z):pass



'''OpenGL extension NV.shader_buffer_store

This module customises the behaviour of the 
OpenGL.raw.GL.NV.shader_buffer_store to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds upon the mechanisms added by the
	NV_shader_buffer_load extension to allow shaders to perform random-access
	reads to buffer object memory without using dedicated buffer object
	binding points.  Instead, it allowed an application to make a buffer
	object resident, query a GPU address (pointer) for the buffer object, and
	then use that address as a pointer in shader code.  This approach allows
	shaders to access a large number of buffer objects without needing to
	repeatedly bind buffers to a limited number of fixed-functionality binding
	points.
	
	This extension lifts the restriction from NV_shader_buffer_load that
	disallows writes.  In particular, the MakeBufferResidentNV function now
	allows READ_WRITE and WRITE_ONLY access modes, and the shading language is
	extended to allow shaders to write through (GPU address) pointers.
	Additionally, the extension provides built-in functions to perform atomic
	memory transactions to buffer object memory.
	
	As with the shader writes provided by the EXT_shader_image_load_store
	extension, writes to buffer object memory using this extension are weakly
	ordered to allow for parallel or distributed shader execution.  The
	EXT_shader_image_load_store extension provides mechanisms allowing for
	finer control of memory transaction order, and those mechanisms apply
	equally to buffer object stores using this extension.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_buffer_store.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.shader_buffer_store import *
from OpenGL.raw.GL.NV.shader_buffer_store import _EXTENSION_NAME

def glInitShaderBufferStoreNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_get_program_binary'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_get_program_binary',error_checker=_errors._error_checker)
GL_NUM_PROGRAM_BINARY_FORMATS=_C('GL_NUM_PROGRAM_BINARY_FORMATS',0x87FE)
GL_PROGRAM_BINARY_FORMATS=_C('GL_PROGRAM_BINARY_FORMATS',0x87FF)
GL_PROGRAM_BINARY_LENGTH=_C('GL_PROGRAM_BINARY_LENGTH',0x8741)
GL_PROGRAM_BINARY_RETRIEVABLE_HINT=_C('GL_PROGRAM_BINARY_RETRIEVABLE_HINT',0x8257)
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLuintArray,ctypes.c_void_p)
def glGetProgramBinary(program,bufSize,length,binaryFormat,binary):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei)
def glProgramBinary(program,binaryFormat,binary,length):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint)
def glProgramParameteri(program,pname,value):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ATI_draw_buffers'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ATI_draw_buffers',error_checker=_errors._error_checker)
GL_DRAW_BUFFER0_ATI=_C('GL_DRAW_BUFFER0_ATI',0x8825)
GL_DRAW_BUFFER10_ATI=_C('GL_DRAW_BUFFER10_ATI',0x882F)
GL_DRAW_BUFFER11_ATI=_C('GL_DRAW_BUFFER11_ATI',0x8830)
GL_DRAW_BUFFER12_ATI=_C('GL_DRAW_BUFFER12_ATI',0x8831)
GL_DRAW_BUFFER13_ATI=_C('GL_DRAW_BUFFER13_ATI',0x8832)
GL_DRAW_BUFFER14_ATI=_C('GL_DRAW_BUFFER14_ATI',0x8833)
GL_DRAW_BUFFER15_ATI=_C('GL_DRAW_BUFFER15_ATI',0x8834)
GL_DRAW_BUFFER1_ATI=_C('GL_DRAW_BUFFER1_ATI',0x8826)
GL_DRAW_BUFFER2_ATI=_C('GL_DRAW_BUFFER2_ATI',0x8827)
GL_DRAW_BUFFER3_ATI=_C('GL_DRAW_BUFFER3_ATI',0x8828)
GL_DRAW_BUFFER4_ATI=_C('GL_DRAW_BUFFER4_ATI',0x8829)
GL_DRAW_BUFFER5_ATI=_C('GL_DRAW_BUFFER5_ATI',0x882A)
GL_DRAW_BUFFER6_ATI=_C('GL_DRAW_BUFFER6_ATI',0x882B)
GL_DRAW_BUFFER7_ATI=_C('GL_DRAW_BUFFER7_ATI',0x882C)
GL_DRAW_BUFFER8_ATI=_C('GL_DRAW_BUFFER8_ATI',0x882D)
GL_DRAW_BUFFER9_ATI=_C('GL_DRAW_BUFFER9_ATI',0x882E)
GL_MAX_DRAW_BUFFERS_ATI=_C('GL_MAX_DRAW_BUFFERS_ATI',0x8824)
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDrawBuffersATI(n,bufs):pass



'''OpenGL extension EXT.texture_compression_s3tc

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_compression_s3tc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional texture compression functionality
	specific to S3's S3TC format (called DXTC in Microsoft's DirectX API),
	subject to all the requirements and limitations described by the extension
	GL_ARB_texture_compression.
	
	This extension supports DXT1, DXT3, and DXT5 texture compression formats.
	For the DXT1 image format, this specification supports an RGB-only mode
	and a special RGBA mode with single-bit "transparent" alpha.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_compression_s3tc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_compression_s3tc import *
from OpenGL.raw.GLES2.EXT.texture_compression_s3tc import _EXTENSION_NAME

def glInitTextureCompressionS3TcEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension KHR.blend_equation_advanced

This module customises the behaviour of the 
OpenGL.raw.GLES2.KHR.blend_equation_advanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a number of "advanced" blending equations that can be
	used to perform new color blending operations, many of which are more
	complex than the standard blend modes provided by unextended OpenGL.  This
	extension provides two different extension string entries:
	
	- KHR_blend_equation_advanced:  Provides the new blending equations, but
	  guarantees defined results only if each sample is touched no more than
	  once in any single rendering pass.  The command BlendBarrierKHR() is
	  provided to indicate a boundary between passes.
	
	- KHR_blend_equation_advanced_coherent:  Provides the new blending
	  equations, and guarantees that blending is done coherently and in API
	  primitive order.  An enable is provided to allow implementations to opt
	  out of fully coherent blending and instead behave as though only
	  KHR_blend_equation_advanced were supported.
	
	Some implementations may support KHR_blend_equation_advanced without
	supporting KHR_blend_equation_advanced_coherent.
	
	In unextended OpenGL, the set of blending equations is limited, and can be
	expressed very simply.  The MIN and MAX blend equations simply compute
	component-wise minimums or maximums of source and destination color
	components.  The FUNC_ADD, FUNC_SUBTRACT, and FUNC_REVERSE_SUBTRACT
	multiply the source and destination colors by source and destination
	factors and either add the two products together or subtract one from the
	other.  This limited set of operations supports many common blending
	operations but precludes the use of more sophisticated transparency and
	blending operations commonly available in many dedicated imaging APIs.
	
	This extension provides a number of new "advanced" blending equations.
	Unlike traditional blending operations using the FUNC_ADD equation, these
	blending equations do not use source and destination factors specified by
	BlendFunc.  Instead, each blend equation specifies a complete equation
	based on the source and destination colors.  These new blend equations are
	used for both RGB and alpha components; they may not be used to perform
	separate RGB and alpha blending (via functions like
	BlendEquationSeparate).
	
	These blending operations are performed using premultiplied source and
	destination colors, where RGB colors produced by the fragment shader and
	stored in the framebuffer are considered to be multiplied by alpha
	(coverage).  Many of these advanced blending equations are formulated
	where the result of blending source and destination colors with partial
	coverage have three separate contributions:  from the portions covered by
	both the source and the destination, from the portion covered only by the
	source, and from the portion covered only by the destination.  Such
	equations are defined assuming that the source and destination coverage
	have no spatial correlation within the pixel.
	
	In addition to the coherency issues on implementations not supporting
	KHR_blend_equation_advanced_coherent, this extension has several
	limitations worth noting.  First, the new blend equations are not
	supported while rendering to more than one color buffer at once; an
	INVALID_OPERATION will be generated if an application attempts to render
	any primitives in this unsupported configuration.  Additionally, blending
	precision may be limited to 16-bit floating-point, which could result in a
	loss of precision and dynamic range for framebuffer formats with 32-bit
	floating-point components, and in a loss of precision for formats with 12-
	and 16-bit signed or unsigned normalized integer components.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/blend_equation_advanced.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.KHR.blend_equation_advanced import *
from OpenGL.raw.GLES2.KHR.blend_equation_advanced import _EXTENSION_NAME

def glInitBlendEquationAdvancedKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_vertex_array'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_vertex_array',error_checker=_errors._error_checker)
GL_COLOR_ARRAY_COUNT_EXT=_C('GL_COLOR_ARRAY_COUNT_EXT',0x8084)
GL_COLOR_ARRAY_EXT=_C('GL_COLOR_ARRAY_EXT',0x8076)
GL_COLOR_ARRAY_POINTER_EXT=_C('GL_COLOR_ARRAY_POINTER_EXT',0x8090)
GL_COLOR_ARRAY_SIZE_EXT=_C('GL_COLOR_ARRAY_SIZE_EXT',0x8081)
GL_COLOR_ARRAY_STRIDE_EXT=_C('GL_COLOR_ARRAY_STRIDE_EXT',0x8083)
GL_COLOR_ARRAY_TYPE_EXT=_C('GL_COLOR_ARRAY_TYPE_EXT',0x8082)
GL_EDGE_FLAG_ARRAY_COUNT_EXT=_C('GL_EDGE_FLAG_ARRAY_COUNT_EXT',0x808D)
GL_EDGE_FLAG_ARRAY_EXT=_C('GL_EDGE_FLAG_ARRAY_EXT',0x8079)
GL_EDGE_FLAG_ARRAY_POINTER_EXT=_C('GL_EDGE_FLAG_ARRAY_POINTER_EXT',0x8093)
GL_EDGE_FLAG_ARRAY_STRIDE_EXT=_C('GL_EDGE_FLAG_ARRAY_STRIDE_EXT',0x808C)
GL_INDEX_ARRAY_COUNT_EXT=_C('GL_INDEX_ARRAY_COUNT_EXT',0x8087)
GL_INDEX_ARRAY_EXT=_C('GL_INDEX_ARRAY_EXT',0x8077)
GL_INDEX_ARRAY_POINTER_EXT=_C('GL_INDEX_ARRAY_POINTER_EXT',0x8091)
GL_INDEX_ARRAY_STRIDE_EXT=_C('GL_INDEX_ARRAY_STRIDE_EXT',0x8086)
GL_INDEX_ARRAY_TYPE_EXT=_C('GL_INDEX_ARRAY_TYPE_EXT',0x8085)
GL_NORMAL_ARRAY_COUNT_EXT=_C('GL_NORMAL_ARRAY_COUNT_EXT',0x8080)
GL_NORMAL_ARRAY_EXT=_C('GL_NORMAL_ARRAY_EXT',0x8075)
GL_NORMAL_ARRAY_POINTER_EXT=_C('GL_NORMAL_ARRAY_POINTER_EXT',0x808F)
GL_NORMAL_ARRAY_STRIDE_EXT=_C('GL_NORMAL_ARRAY_STRIDE_EXT',0x807F)
GL_NORMAL_ARRAY_TYPE_EXT=_C('GL_NORMAL_ARRAY_TYPE_EXT',0x807E)
GL_TEXTURE_COORD_ARRAY_COUNT_EXT=_C('GL_TEXTURE_COORD_ARRAY_COUNT_EXT',0x808B)
GL_TEXTURE_COORD_ARRAY_EXT=_C('GL_TEXTURE_COORD_ARRAY_EXT',0x8078)
GL_TEXTURE_COORD_ARRAY_POINTER_EXT=_C('GL_TEXTURE_COORD_ARRAY_POINTER_EXT',0x8092)
GL_TEXTURE_COORD_ARRAY_SIZE_EXT=_C('GL_TEXTURE_COORD_ARRAY_SIZE_EXT',0x8088)
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT=_C('GL_TEXTURE_COORD_ARRAY_STRIDE_EXT',0x808A)
GL_TEXTURE_COORD_ARRAY_TYPE_EXT=_C('GL_TEXTURE_COORD_ARRAY_TYPE_EXT',0x8089)
GL_VERTEX_ARRAY_COUNT_EXT=_C('GL_VERTEX_ARRAY_COUNT_EXT',0x807D)
GL_VERTEX_ARRAY_EXT=_C('GL_VERTEX_ARRAY_EXT',0x8074)
GL_VERTEX_ARRAY_POINTER_EXT=_C('GL_VERTEX_ARRAY_POINTER_EXT',0x808E)
GL_VERTEX_ARRAY_SIZE_EXT=_C('GL_VERTEX_ARRAY_SIZE_EXT',0x807A)
GL_VERTEX_ARRAY_STRIDE_EXT=_C('GL_VERTEX_ARRAY_STRIDE_EXT',0x807C)
GL_VERTEX_ARRAY_TYPE_EXT=_C('GL_VERTEX_ARRAY_TYPE_EXT',0x807B)
@_f
@_p.types(None,_cs.GLint)
def glArrayElementEXT(i):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,ctypes.c_void_p)
def glColorPointerEXT(size,type,stride,count,pointer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLsizei)
def glDrawArraysEXT(mode,first,count):pass
@_f
@_p.types(None,_cs.GLsizei,_cs.GLsizei,arrays.GLbooleanArray)
def glEdgeFlagPointerEXT(stride,count,pointer):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLvoidpArray)
def glGetPointervEXT(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,ctypes.c_void_p)
def glIndexPointerEXT(type,stride,count,pointer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,ctypes.c_void_p)
def glNormalPointerEXT(type,stride,count,pointer):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,ctypes.c_void_p)
def glTexCoordPointerEXT(size,type,stride,count,pointer):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,ctypes.c_void_p)
def glVertexPointerEXT(size,type,stride,count,pointer):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_4_0'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_4_0',error_checker=_errors._error_checker)
GL_ACTIVE_SUBROUTINES=_C('GL_ACTIVE_SUBROUTINES',0x8DE5)
GL_ACTIVE_SUBROUTINE_MAX_LENGTH=_C('GL_ACTIVE_SUBROUTINE_MAX_LENGTH',0x8E48)
GL_ACTIVE_SUBROUTINE_UNIFORMS=_C('GL_ACTIVE_SUBROUTINE_UNIFORMS',0x8DE6)
GL_ACTIVE_SUBROUTINE_UNIFORM_LOCATIONS=_C('GL_ACTIVE_SUBROUTINE_UNIFORM_LOCATIONS',0x8E47)
GL_ACTIVE_SUBROUTINE_UNIFORM_MAX_LENGTH=_C('GL_ACTIVE_SUBROUTINE_UNIFORM_MAX_LENGTH',0x8E49)
GL_COMPATIBLE_SUBROUTINES=_C('GL_COMPATIBLE_SUBROUTINES',0x8E4B)
GL_DOUBLE_MAT2=_C('GL_DOUBLE_MAT2',0x8F46)
GL_DOUBLE_MAT2x3=_C('GL_DOUBLE_MAT2x3',0x8F49)
GL_DOUBLE_MAT2x4=_C('GL_DOUBLE_MAT2x4',0x8F4A)
GL_DOUBLE_MAT3=_C('GL_DOUBLE_MAT3',0x8F47)
GL_DOUBLE_MAT3x2=_C('GL_DOUBLE_MAT3x2',0x8F4B)
GL_DOUBLE_MAT3x4=_C('GL_DOUBLE_MAT3x4',0x8F4C)
GL_DOUBLE_MAT4=_C('GL_DOUBLE_MAT4',0x8F48)
GL_DOUBLE_MAT4x2=_C('GL_DOUBLE_MAT4x2',0x8F4D)
GL_DOUBLE_MAT4x3=_C('GL_DOUBLE_MAT4x3',0x8F4E)
GL_DOUBLE_VEC2=_C('GL_DOUBLE_VEC2',0x8FFC)
GL_DOUBLE_VEC3=_C('GL_DOUBLE_VEC3',0x8FFD)
GL_DOUBLE_VEC4=_C('GL_DOUBLE_VEC4',0x8FFE)
GL_DRAW_INDIRECT_BUFFER=_C('GL_DRAW_INDIRECT_BUFFER',0x8F3F)
GL_DRAW_INDIRECT_BUFFER_BINDING=_C('GL_DRAW_INDIRECT_BUFFER_BINDING',0x8F43)
GL_FRACTIONAL_EVEN=_C('GL_FRACTIONAL_EVEN',0x8E7C)
GL_FRACTIONAL_ODD=_C('GL_FRACTIONAL_ODD',0x8E7B)
GL_FRAGMENT_INTERPOLATION_OFFSET_BITS=_C('GL_FRAGMENT_INTERPOLATION_OFFSET_BITS',0x8E5D)
GL_GEOMETRY_SHADER_INVOCATIONS=_C('GL_GEOMETRY_SHADER_INVOCATIONS',0x887F)
GL_INT_SAMPLER_CUBE_MAP_ARRAY=_C('GL_INT_SAMPLER_CUBE_MAP_ARRAY',0x900E)
GL_ISOLINES=_C('GL_ISOLINES',0x8E7A)
GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS',0x8E1E)
GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS',0x8E1F)
GL_MAX_FRAGMENT_INTERPOLATION_OFFSET=_C('GL_MAX_FRAGMENT_INTERPOLATION_OFFSET',0x8E5C)
GL_MAX_GEOMETRY_SHADER_INVOCATIONS=_C('GL_MAX_GEOMETRY_SHADER_INVOCATIONS',0x8E5A)
GL_MAX_PATCH_VERTICES=_C('GL_MAX_PATCH_VERTICES',0x8E7D)
GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET=_C('GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET',0x8E5F)
GL_MAX_SUBROUTINES=_C('GL_MAX_SUBROUTINES',0x8DE7)
GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS=_C('GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS',0x8DE8)
GL_MAX_TESS_CONTROL_INPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_INPUT_COMPONENTS',0x886C)
GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS',0x8E83)
GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS',0x8E81)
GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS',0x8E85)
GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS=_C('GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS',0x8E89)
GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS=_C('GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS',0x8E7F)
GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS',0x886D)
GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS',0x8E86)
GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS',0x8E82)
GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS=_C('GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS',0x8E8A)
GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS',0x8E80)
GL_MAX_TESS_GEN_LEVEL=_C('GL_MAX_TESS_GEN_LEVEL',0x8E7E)
GL_MAX_TESS_PATCH_COMPONENTS=_C('GL_MAX_TESS_PATCH_COMPONENTS',0x8E84)
GL_MAX_TRANSFORM_FEEDBACK_BUFFERS=_C('GL_MAX_TRANSFORM_FEEDBACK_BUFFERS',0x8E70)
GL_MAX_VERTEX_STREAMS=_C('GL_MAX_VERTEX_STREAMS',0x8E71)
GL_MAX_VERTEX_STREAMS=_C('GL_MAX_VERTEX_STREAMS',0x8E71)
GL_MIN_FRAGMENT_INTERPOLATION_OFFSET=_C('GL_MIN_FRAGMENT_INTERPOLATION_OFFSET',0x8E5B)
GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET=_C('GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET',0x8E5E)
GL_MIN_SAMPLE_SHADING_VALUE=_C('GL_MIN_SAMPLE_SHADING_VALUE',0x8C37)
GL_NUM_COMPATIBLE_SUBROUTINES=_C('GL_NUM_COMPATIBLE_SUBROUTINES',0x8E4A)
GL_PATCHES=_C('GL_PATCHES',0x000E)
GL_PATCH_DEFAULT_INNER_LEVEL=_C('GL_PATCH_DEFAULT_INNER_LEVEL',0x8E73)
GL_PATCH_DEFAULT_OUTER_LEVEL=_C('GL_PATCH_DEFAULT_OUTER_LEVEL',0x8E74)
GL_PATCH_VERTICES=_C('GL_PATCH_VERTICES',0x8E72)
GL_PROXY_TEXTURE_CUBE_MAP_ARRAY=_C('GL_PROXY_TEXTURE_CUBE_MAP_ARRAY',0x900B)
GL_QUADS=_C('GL_QUADS',0x0007)
GL_SAMPLER_CUBE_MAP_ARRAY=_C('GL_SAMPLER_CUBE_MAP_ARRAY',0x900C)
GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW=_C('GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW',0x900D)
GL_SAMPLE_SHADING=_C('GL_SAMPLE_SHADING',0x8C36)
GL_TESS_CONTROL_OUTPUT_VERTICES=_C('GL_TESS_CONTROL_OUTPUT_VERTICES',0x8E75)
GL_TESS_CONTROL_SHADER=_C('GL_TESS_CONTROL_SHADER',0x8E88)
GL_TESS_EVALUATION_SHADER=_C('GL_TESS_EVALUATION_SHADER',0x8E87)
GL_TESS_GEN_MODE=_C('GL_TESS_GEN_MODE',0x8E76)
GL_TESS_GEN_POINT_MODE=_C('GL_TESS_GEN_POINT_MODE',0x8E79)
GL_TESS_GEN_SPACING=_C('GL_TESS_GEN_SPACING',0x8E77)
GL_TESS_GEN_VERTEX_ORDER=_C('GL_TESS_GEN_VERTEX_ORDER',0x8E78)
GL_TEXTURE_BINDING_CUBE_MAP_ARRAY=_C('GL_TEXTURE_BINDING_CUBE_MAP_ARRAY',0x900A)
GL_TEXTURE_CUBE_MAP_ARRAY=_C('GL_TEXTURE_CUBE_MAP_ARRAY',0x9009)
GL_TRANSFORM_FEEDBACK=_C('GL_TRANSFORM_FEEDBACK',0x8E22)
GL_TRANSFORM_FEEDBACK_BINDING=_C('GL_TRANSFORM_FEEDBACK_BINDING',0x8E25)
GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE=_C('GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE',0x8E24)
GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED=_C('GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED',0x8E23)
GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_CONTROL_SHADER=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_CONTROL_SHADER',0x84F0)
GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_EVALUATION_SHADER=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_EVALUATION_SHADER',0x84F1)
GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY=_C('GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY',0x900F)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint)
def glBeginQueryIndexed(target,index,id):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindTransformFeedback(target,id):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum)
def glBlendEquationSeparatei(buf,modeRGB,modeAlpha):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glBlendEquationi(buf,mode):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glBlendFuncSeparatei(buf,srcRGB,dstRGB,srcAlpha,dstAlpha):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum)
def glBlendFunci(buf,src,dst):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteTransformFeedbacks(n,ids):pass
@_f
@_p.types(None,_cs.GLenum,ctypes.c_void_p)
def glDrawArraysIndirect(mode,indirect):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glDrawElementsIndirect(mode,type,indirect):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDrawTransformFeedback(mode,id):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint)
def glDrawTransformFeedbackStream(mode,id,stream):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEndQueryIndexed(target,index):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenTransformFeedbacks(n,ids):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetActiveSubroutineName(program,shadertype,index,bufsize,length,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetActiveSubroutineUniformName(program,shadertype,index,bufsize,length,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetActiveSubroutineUniformiv(program,shadertype,index,pname,values):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetProgramStageiv(program,shadertype,pname,values):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetQueryIndexediv(target,index,pname,params):pass
@_f
@_p.types(_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetSubroutineIndex(program,shadertype,name):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetSubroutineUniformLocation(program,shadertype,name):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,arrays.GLuintArray)
def glGetUniformSubroutineuiv(shadertype,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLdoubleArray)
def glGetUniformdv(program,location,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsTransformFeedback(id):pass
@_f
@_p.types(None,_cs.GLfloat)
def glMinSampleShading(value):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glPatchParameterfv(pname,values):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glPatchParameteri(pname,value):pass
@_f
@_p.types(None,)
def glPauseTransformFeedback():pass
@_f
@_p.types(None,)
def glResumeTransformFeedback():pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble)
def glUniform1d(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform1dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble)
def glUniform2d(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform2dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glUniform3d(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform3dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glUniform4d(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform4dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2x3dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2x4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3x2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3x4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4x2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4x3dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray)
def glUniformSubroutinesuiv(shadertype,count,indices):pass



'''OpenGL extension VERSION.GL_4_4

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_4_4 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_4_4.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_4_4 import *
from OpenGL.raw.GL.VERSION.GL_4_4 import _EXTENSION_NAME

def glInitGl44VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glBufferStorage.data size not checked against size
glBufferStorage=wrapper.wrapper(glBufferStorage).setInputArraySize(
    'data', None
)
# INPUT glClearTexImage.data size not checked against 'format,type'
glClearTexImage=wrapper.wrapper(glClearTexImage).setInputArraySize(
    'data', None
)
# INPUT glClearTexSubImage.data size not checked against 'format,type'
glClearTexSubImage=wrapper.wrapper(glClearTexSubImage).setInputArraySize(
    'data', None
)
# INPUT glBindBuffersBase.buffers size not checked against count
glBindBuffersBase=wrapper.wrapper(glBindBuffersBase).setInputArraySize(
    'buffers', None
)
# INPUT glBindBuffersRange.buffers size not checked against count
# INPUT glBindBuffersRange.offsets size not checked against count
# INPUT glBindBuffersRange.sizes size not checked against count
glBindBuffersRange=wrapper.wrapper(glBindBuffersRange).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'offsets', None
).setInputArraySize(
    'sizes', None
)
# INPUT glBindTextures.textures size not checked against count
glBindTextures=wrapper.wrapper(glBindTextures).setInputArraySize(
    'textures', None
)
# INPUT glBindSamplers.samplers size not checked against count
glBindSamplers=wrapper.wrapper(glBindSamplers).setInputArraySize(
    'samplers', None
)
# INPUT glBindImageTextures.textures size not checked against count
glBindImageTextures=wrapper.wrapper(glBindImageTextures).setInputArraySize(
    'textures', None
)
# INPUT glBindVertexBuffers.buffers size not checked against count
# INPUT glBindVertexBuffers.offsets size not checked against count
# INPUT glBindVertexBuffers.strides size not checked against count
glBindVertexBuffers=wrapper.wrapper(glBindVertexBuffers).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'offsets', None
).setInputArraySize(
    'strides', None
)
### END AUTOGENERATED SECTION



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_I3D_genlock'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_I3D_genlock',error_checker=_errors._error_checker)
WGL_GENLOCK_SOURCE_DIGITAL_FIELD_I3D=_C('WGL_GENLOCK_SOURCE_DIGITAL_FIELD_I3D',0x2049)
WGL_GENLOCK_SOURCE_DIGITAL_SYNC_I3D=_C('WGL_GENLOCK_SOURCE_DIGITAL_SYNC_I3D',0x2048)
WGL_GENLOCK_SOURCE_EDGE_BOTH_I3D=_C('WGL_GENLOCK_SOURCE_EDGE_BOTH_I3D',0x204C)
WGL_GENLOCK_SOURCE_EDGE_FALLING_I3D=_C('WGL_GENLOCK_SOURCE_EDGE_FALLING_I3D',0x204A)
WGL_GENLOCK_SOURCE_EDGE_RISING_I3D=_C('WGL_GENLOCK_SOURCE_EDGE_RISING_I3D',0x204B)
WGL_GENLOCK_SOURCE_EXTERNAL_FIELD_I3D=_C('WGL_GENLOCK_SOURCE_EXTERNAL_FIELD_I3D',0x2046)
WGL_GENLOCK_SOURCE_EXTERNAL_SYNC_I3D=_C('WGL_GENLOCK_SOURCE_EXTERNAL_SYNC_I3D',0x2045)
WGL_GENLOCK_SOURCE_EXTERNAL_TTL_I3D=_C('WGL_GENLOCK_SOURCE_EXTERNAL_TTL_I3D',0x2047)
WGL_GENLOCK_SOURCE_MULTIVIEW_I3D=_C('WGL_GENLOCK_SOURCE_MULTIVIEW_I3D',0x2044)
@_f
@_p.types(_cs.BOOL,_cs.HDC)
def wglDisableGenlockI3D(hDC):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC)
def wglEnableGenlockI3D(hDC):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.UINT)
def wglGenlockSampleRateI3D(hDC,uRate):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.UINT)
def wglGenlockSourceDelayI3D(hDC,uDelay):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.UINT)
def wglGenlockSourceEdgeI3D(hDC,uEdge):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.UINT)
def wglGenlockSourceI3D(hDC,uSource):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.UINT))
def wglGetGenlockSampleRateI3D(hDC,uRate):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.UINT))
def wglGetGenlockSourceDelayI3D(hDC,uDelay):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.UINT))
def wglGetGenlockSourceEdgeI3D(hDC,uEdge):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.UINT))
def wglGetGenlockSourceI3D(hDC,uSource):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.BOOL))
def wglIsEnabledGenlockI3D(hDC,pFlag):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.UINT),ctypes.POINTER(_cs.UINT))
def wglQueryGenlockMaxSourceDelayI3D(hDC,uMaxLineDelay,uMaxPixelDelay):pass



'''OpenGL extension EXT.texture_compression_dxt1

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_compression_dxt1 to provide a more 
Python-friendly API

Overview (from the spec)
	
	Support of EXT_texture_compression_s3tc is attractive for OpenGL-ES
	implementations because it provides compressed textures that allow
	for significantly reduced texture storage. Reducing texture storage is 
	advantageous because of the smaller memory capacity of many embedded 
	systems compared to desktop systems. Smaller textures also provide a
	welcome performance advantage since embedded platforms typically provide
	less performance than desktop systems. S3TC compressed textures 
	are widely supported and used by applications. The DXT1 format is 
	used in the vast majority of cases in which S3TC compressed textures 
	are used.
	
	However, EXT_texture_compression_s3tc specifies functionality that is
	burdensome for an OpenGL-ES implementation. In particular it requires
	that the driver provide the capability to compress textures into 
	S3TC texture formats, as an S3TC texture format is accepted as the
	<internalformat> parameter of TexImage2D and CopyTexImage2D. Further,
	EXT_texture_compression_s3tc may require conversion from one S3TC 
	format to another during CompressedTexSubImage2D if the <format> 
	parameter does not match the <internalformat> of the texture image 
	previously created by TexImage2D.
	
	In an OpenGL-ES implementation it is therefore advantageous to support 
	a limited subset of EXT_texture_compression_s3tc: Restrict supported 
	texture formats to DXT1 and restrict supported operations to those
	that do not require texture compression into an S3TC texture format or
	decompression from an S3TC texture format.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_compression_dxt1.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_compression_dxt1 import *
from OpenGL.raw.GLES2.EXT.texture_compression_dxt1 import _EXTENSION_NAME

def glInitTextureCompressionDxt1EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_border_clamp

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_border_clamp to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_border_clamp.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_border_clamp import *
from OpenGL.raw.GLES2.EXT.texture_border_clamp import _EXTENSION_NAME

def glInitTextureBorderClampEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glTexParameterIivEXT.params size not checked against 'pname'
glTexParameterIivEXT=wrapper.wrapper(glTexParameterIivEXT).setInputArraySize(
    'params', None
)
# INPUT glTexParameterIuivEXT.params size not checked against 'pname'
glTexParameterIuivEXT=wrapper.wrapper(glTexParameterIuivEXT).setInputArraySize(
    'params', None
)
glGetTexParameterIivEXT=wrapper.wrapper(glGetTexParameterIivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexParameterIuivEXT=wrapper.wrapper(glGetTexParameterIuivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glSamplerParameterIivEXT.param size not checked against 'pname'
glSamplerParameterIivEXT=wrapper.wrapper(glSamplerParameterIivEXT).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterIuivEXT.param size not checked against 'pname'
glSamplerParameterIuivEXT=wrapper.wrapper(glSamplerParameterIuivEXT).setInputArraySize(
    'param', None
)
# INPUT glGetSamplerParameterIivEXT.params size not checked against 'pname'
glGetSamplerParameterIivEXT=wrapper.wrapper(glGetSamplerParameterIivEXT).setInputArraySize(
    'params', None
)
# INPUT glGetSamplerParameterIuivEXT.params size not checked against 'pname'
glGetSamplerParameterIuivEXT=wrapper.wrapper(glGetSamplerParameterIuivEXT).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_polygon_offset'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_polygon_offset',error_checker=_errors._error_checker)
GL_POLYGON_OFFSET_BIAS_EXT=_C('GL_POLYGON_OFFSET_BIAS_EXT',0x8039)
GL_POLYGON_OFFSET_EXT=_C('GL_POLYGON_OFFSET_EXT',0x8037)
GL_POLYGON_OFFSET_FACTOR_EXT=_C('GL_POLYGON_OFFSET_FACTOR_EXT',0x8038)
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glPolygonOffsetEXT(factor,bias):pass



'''OpenGL extension ARB.transform_feedback_instanced

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.transform_feedback_instanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	Multiple instances of geometry may be specified to the GL by calling
	functions such as DrawArraysInstanced and DrawElementsInstanced. Further,
	the results of a transform feedback operation may be returned to the GL
	by calling DrawTransformFeedback, or DrawTransformFeedbackStream. However,
	it is not presently possible to draw multiple instances of data
	transform feedback without using a query and the resulting round trip from
	server to client.
	
	This extension adds functionality to draw multiple instances of the result
	of a transform feedback operation.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/transform_feedback_instanced.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.transform_feedback_instanced import *
from OpenGL.raw.GL.ARB.transform_feedback_instanced import _EXTENSION_NAME

def glInitTransformFeedbackInstancedARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIX_list_priority'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIX_list_priority',error_checker=_errors._error_checker)
GL_LIST_PRIORITY_SGIX=_C('GL_LIST_PRIORITY_SGIX',0x8182)
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glGetListParameterfvSGIX(list,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetListParameterivSGIX(list,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLfloat)
def glListParameterfSGIX(list,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glListParameterfvSGIX(list,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint)
def glListParameteriSGIX(list,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glListParameterivSGIX(list,pname,params):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_texture_array'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_texture_array',error_checker=_errors._error_checker)
GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT=_C('GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT',0x884E)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT',0x8CD4)
GL_MAX_ARRAY_TEXTURE_LAYERS_EXT=_C('GL_MAX_ARRAY_TEXTURE_LAYERS_EXT',0x88FF)
GL_PROXY_TEXTURE_1D_ARRAY_EXT=_C('GL_PROXY_TEXTURE_1D_ARRAY_EXT',0x8C19)
GL_PROXY_TEXTURE_2D_ARRAY_EXT=_C('GL_PROXY_TEXTURE_2D_ARRAY_EXT',0x8C1B)
GL_TEXTURE_1D_ARRAY_EXT=_C('GL_TEXTURE_1D_ARRAY_EXT',0x8C18)
GL_TEXTURE_2D_ARRAY_EXT=_C('GL_TEXTURE_2D_ARRAY_EXT',0x8C1A)
GL_TEXTURE_BINDING_1D_ARRAY_EXT=_C('GL_TEXTURE_BINDING_1D_ARRAY_EXT',0x8C1C)
GL_TEXTURE_BINDING_2D_ARRAY_EXT=_C('GL_TEXTURE_BINDING_2D_ARRAY_EXT',0x8C1D)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint)
def glFramebufferTextureLayerEXT(target,attachment,texture,level,layer):pass



'''OpenGL extension NV.shader_buffer_load

This module customises the behaviour of the 
OpenGL.raw.GL.NV.shader_buffer_load to provide a more 
Python-friendly API

Overview (from the spec)
	
	At a very coarse level, GL has evolved in a way that allows 
	applications to replace many of the original state machine variables 
	with blocks of user-defined data. For example, the current vertex 
	state has been augmented by vertex buffer objects, fixed-function 
	shading state and parameters have been replaced by shaders/programs 
	and constant buffers, etc.. Applications switch between coarse sets 
	of state by binding objects to the context or to other container 
	objects (e.g. vertex array objects) instead of manipulating state 
	variables of the context. In terms of the number of GL commands 
	required to draw an object, modern applications are orders of 
	magnitude more efficient than legacy applications, but this explosion 
	of objects bound to other objects has led to a new bottleneck - 
	pointer chasing and CPU L2 cache misses in the driver, and general 
	L2 cache pollution.
	
	This extension provides a mechanism to read from a flat, 64-bit GPU 
	address space from programs/shaders, to query GPU addresses of buffer
	objects at the API level, and to bind buffer objects to the context in
	such a way that they can be accessed via their GPU addresses in any 
	shader stage. 
	
	The intent is that applications can avoid re-binding buffer objects 
	or updating constants between each Draw call and instead simply use 
	a VertexAttrib (or TexCoord, or InstanceID, or...) to "point" to the 
	new object's state. In this way, one of the cheapest "state" updates 
	(from the CPU's point of view) can be used to effect a significant 
	state change in the shader similarly to how a pointer change may on 
	the CPU. At the same time, this relieves the limits on how many 
	buffer objects can be accessed at once by shaders, and allows these 
	buffer object accesses to be exposed as C-style pointer dereferences
	in the shading language.
	
	As a very simple example, imagine packing a group of similar objects' 
	constants into a single buffer object and pointing your program
	at object <i> by setting "glVertexAttribI1iEXT(attrLoc, i);"
	and using a shader as such:
	
	    struct MyObjectType {
	        mat4x4 modelView;
	        vec4 materialPropertyX;
	        // etc.
	    };
	    uniform MyObjectType *allObjects;
	    in int objectID; // bound to attrLoc
	
	    ...
	
	    mat4x4 thisObjectsMatrix = allObjects[objectID].modelView;
	    // do transform, shading, etc.
	
	This is beneficial in much the same way that texture arrays allow 
	choosing between similar, but independent, texture maps with a single
	coordinate identifying which slice of the texture to use. It also
	resembles instancing, where a lightweight change (incrementing the 
	instance ID) can be used to generate a different and interesting 
	result, but with additional flexibility over instancing because the 
	values are app-controlled and not a single incrementing counter.
	
	Dependent pointer fetches are allowed, so more complex scene graph 
	structures can be built into buffer objects providing significant new 
	flexibility in the use of shaders. Another simple example, showing 
	something you can't do with existing functionality, is to do dependent
	fetches into many buffer objects:
	
	    GenBuffers(N, dataBuffers);
	    GenBuffers(1, &pointerBuffer);
	
	    GLuint64EXT gpuAddrs[N];
	    for (i = 0; i < N; ++i) {
	        BindBuffer(target, dataBuffers[i]);
	        BufferData(target, size[i], myData[i], STATIC_DRAW);
	
	        // get the address of this buffer and make it resident.
	        GetBufferParameterui64vNV(target, BUFFER_GPU_ADDRESS, 
	                                  gpuaddrs[i]); 
	        MakeBufferResidentNV(target, READ_ONLY);
	    }
	
	    GLuint64EXT pointerBufferAddr;
	    BindBuffer(target, pointerBuffer);
	    BufferData(target, sizeof(GLuint64EXT)*N, gpuAddrs, STATIC_DRAW);
	    GetBufferParameterui64vNV(target, BUFFER_GPU_ADDRESS, 
	                              &pointerBufferAddr); 
	    MakeBufferResidentNV(target, READ_ONLY);
	
	    // now in the shader, we can use a double indirection
	    vec4 **ptrToBuffers = pointerBufferAddr;
	    vec4 *ptrToBufferI = ptrToBuffers[i];
	
	This allows simultaneous access to more buffers than 
	EXT_bindable_uniform (MAX_VERTEX_BINDABLE_UNIFORMS, etc.) and each
	can be larger than MAX_BINDABLE_UNIFORM_SIZE.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_buffer_load.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.shader_buffer_load import *
from OpenGL.raw.GL.NV.shader_buffer_load import _EXTENSION_NAME

def glInitShaderBufferLoadNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetBufferParameterui64vNV=wrapper.wrapper(glGetBufferParameterui64vNV).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetNamedBufferParameterui64vNV=wrapper.wrapper(glGetNamedBufferParameterui64vNV).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetIntegerui64vNV=wrapper.wrapper(glGetIntegerui64vNV).setOutput(
    'result',size=_glgets._glget_size_mapping,pnameArg='value',orPassIn=True
)
# INPUT glUniformui64vNV.value size not checked against count
glUniformui64vNV=wrapper.wrapper(glUniformui64vNV).setInputArraySize(
    'value', None
)
# OUTPUT glGetUniformui64vNV.params COMPSIZE(program, location) 
# INPUT glProgramUniformui64vNV.value size not checked against count
glProgramUniformui64vNV=wrapper.wrapper(glProgramUniformui64vNV).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_1_3'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_1_3',error_checker=_errors._error_checker)
GL_ACTIVE_TEXTURE=_C('GL_ACTIVE_TEXTURE',0x84E0)
GL_ADD_SIGNED=_C('GL_ADD_SIGNED',0x8574)
GL_CLAMP_TO_BORDER=_C('GL_CLAMP_TO_BORDER',0x812D)
GL_CLIENT_ACTIVE_TEXTURE=_C('GL_CLIENT_ACTIVE_TEXTURE',0x84E1)
GL_COMBINE=_C('GL_COMBINE',0x8570)
GL_COMBINE_ALPHA=_C('GL_COMBINE_ALPHA',0x8572)
GL_COMBINE_RGB=_C('GL_COMBINE_RGB',0x8571)
GL_COMPRESSED_ALPHA=_C('GL_COMPRESSED_ALPHA',0x84E9)
GL_COMPRESSED_INTENSITY=_C('GL_COMPRESSED_INTENSITY',0x84EC)
GL_COMPRESSED_LUMINANCE=_C('GL_COMPRESSED_LUMINANCE',0x84EA)
GL_COMPRESSED_LUMINANCE_ALPHA=_C('GL_COMPRESSED_LUMINANCE_ALPHA',0x84EB)
GL_COMPRESSED_RGB=_C('GL_COMPRESSED_RGB',0x84ED)
GL_COMPRESSED_RGBA=_C('GL_COMPRESSED_RGBA',0x84EE)
GL_COMPRESSED_TEXTURE_FORMATS=_C('GL_COMPRESSED_TEXTURE_FORMATS',0x86A3)
GL_CONSTANT=_C('GL_CONSTANT',0x8576)
GL_DOT3_RGB=_C('GL_DOT3_RGB',0x86AE)
GL_DOT3_RGBA=_C('GL_DOT3_RGBA',0x86AF)
GL_INTERPOLATE=_C('GL_INTERPOLATE',0x8575)
GL_MAX_CUBE_MAP_TEXTURE_SIZE=_C('GL_MAX_CUBE_MAP_TEXTURE_SIZE',0x851C)
GL_MAX_TEXTURE_UNITS=_C('GL_MAX_TEXTURE_UNITS',0x84E2)
GL_MULTISAMPLE=_C('GL_MULTISAMPLE',0x809D)
GL_MULTISAMPLE_BIT=_C('GL_MULTISAMPLE_BIT',0x20000000)
GL_NORMAL_MAP=_C('GL_NORMAL_MAP',0x8511)
GL_NUM_COMPRESSED_TEXTURE_FORMATS=_C('GL_NUM_COMPRESSED_TEXTURE_FORMATS',0x86A2)
GL_OPERAND0_ALPHA=_C('GL_OPERAND0_ALPHA',0x8598)
GL_OPERAND0_RGB=_C('GL_OPERAND0_RGB',0x8590)
GL_OPERAND1_ALPHA=_C('GL_OPERAND1_ALPHA',0x8599)
GL_OPERAND1_RGB=_C('GL_OPERAND1_RGB',0x8591)
GL_OPERAND2_ALPHA=_C('GL_OPERAND2_ALPHA',0x859A)
GL_OPERAND2_RGB=_C('GL_OPERAND2_RGB',0x8592)
GL_PREVIOUS=_C('GL_PREVIOUS',0x8578)
GL_PRIMARY_COLOR=_C('GL_PRIMARY_COLOR',0x8577)
GL_PROXY_TEXTURE_CUBE_MAP=_C('GL_PROXY_TEXTURE_CUBE_MAP',0x851B)
GL_REFLECTION_MAP=_C('GL_REFLECTION_MAP',0x8512)
GL_RGB_SCALE=_C('GL_RGB_SCALE',0x8573)
GL_SAMPLES=_C('GL_SAMPLES',0x80A9)
GL_SAMPLE_ALPHA_TO_COVERAGE=_C('GL_SAMPLE_ALPHA_TO_COVERAGE',0x809E)
GL_SAMPLE_ALPHA_TO_ONE=_C('GL_SAMPLE_ALPHA_TO_ONE',0x809F)
GL_SAMPLE_BUFFERS=_C('GL_SAMPLE_BUFFERS',0x80A8)
GL_SAMPLE_COVERAGE=_C('GL_SAMPLE_COVERAGE',0x80A0)
GL_SAMPLE_COVERAGE_INVERT=_C('GL_SAMPLE_COVERAGE_INVERT',0x80AB)
GL_SAMPLE_COVERAGE_VALUE=_C('GL_SAMPLE_COVERAGE_VALUE',0x80AA)
GL_SOURCE0_ALPHA=_C('GL_SOURCE0_ALPHA',0x8588)
GL_SOURCE0_RGB=_C('GL_SOURCE0_RGB',0x8580)
GL_SOURCE1_ALPHA=_C('GL_SOURCE1_ALPHA',0x8589)
GL_SOURCE1_RGB=_C('GL_SOURCE1_RGB',0x8581)
GL_SOURCE2_ALPHA=_C('GL_SOURCE2_ALPHA',0x858A)
GL_SOURCE2_RGB=_C('GL_SOURCE2_RGB',0x8582)
GL_SUBTRACT=_C('GL_SUBTRACT',0x84E7)
GL_TEXTURE0=_C('GL_TEXTURE0',0x84C0)
GL_TEXTURE1=_C('GL_TEXTURE1',0x84C1)
GL_TEXTURE10=_C('GL_TEXTURE10',0x84CA)
GL_TEXTURE11=_C('GL_TEXTURE11',0x84CB)
GL_TEXTURE12=_C('GL_TEXTURE12',0x84CC)
GL_TEXTURE13=_C('GL_TEXTURE13',0x84CD)
GL_TEXTURE14=_C('GL_TEXTURE14',0x84CE)
GL_TEXTURE15=_C('GL_TEXTURE15',0x84CF)
GL_TEXTURE16=_C('GL_TEXTURE16',0x84D0)
GL_TEXTURE17=_C('GL_TEXTURE17',0x84D1)
GL_TEXTURE18=_C('GL_TEXTURE18',0x84D2)
GL_TEXTURE19=_C('GL_TEXTURE19',0x84D3)
GL_TEXTURE2=_C('GL_TEXTURE2',0x84C2)
GL_TEXTURE20=_C('GL_TEXTURE20',0x84D4)
GL_TEXTURE21=_C('GL_TEXTURE21',0x84D5)
GL_TEXTURE22=_C('GL_TEXTURE22',0x84D6)
GL_TEXTURE23=_C('GL_TEXTURE23',0x84D7)
GL_TEXTURE24=_C('GL_TEXTURE24',0x84D8)
GL_TEXTURE25=_C('GL_TEXTURE25',0x84D9)
GL_TEXTURE26=_C('GL_TEXTURE26',0x84DA)
GL_TEXTURE27=_C('GL_TEXTURE27',0x84DB)
GL_TEXTURE28=_C('GL_TEXTURE28',0x84DC)
GL_TEXTURE29=_C('GL_TEXTURE29',0x84DD)
GL_TEXTURE3=_C('GL_TEXTURE3',0x84C3)
GL_TEXTURE30=_C('GL_TEXTURE30',0x84DE)
GL_TEXTURE31=_C('GL_TEXTURE31',0x84DF)
GL_TEXTURE4=_C('GL_TEXTURE4',0x84C4)
GL_TEXTURE5=_C('GL_TEXTURE5',0x84C5)
GL_TEXTURE6=_C('GL_TEXTURE6',0x84C6)
GL_TEXTURE7=_C('GL_TEXTURE7',0x84C7)
GL_TEXTURE8=_C('GL_TEXTURE8',0x84C8)
GL_TEXTURE9=_C('GL_TEXTURE9',0x84C9)
GL_TEXTURE_BINDING_CUBE_MAP=_C('GL_TEXTURE_BINDING_CUBE_MAP',0x8514)
GL_TEXTURE_COMPRESSED=_C('GL_TEXTURE_COMPRESSED',0x86A1)
GL_TEXTURE_COMPRESSED_IMAGE_SIZE=_C('GL_TEXTURE_COMPRESSED_IMAGE_SIZE',0x86A0)
GL_TEXTURE_COMPRESSION_HINT=_C('GL_TEXTURE_COMPRESSION_HINT',0x84EF)
GL_TEXTURE_CUBE_MAP=_C('GL_TEXTURE_CUBE_MAP',0x8513)
GL_TEXTURE_CUBE_MAP_NEGATIVE_X=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_X',0x8516)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_Y',0x8518)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_Z',0x851A)
GL_TEXTURE_CUBE_MAP_POSITIVE_X=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_X',0x8515)
GL_TEXTURE_CUBE_MAP_POSITIVE_Y=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_Y',0x8517)
GL_TEXTURE_CUBE_MAP_POSITIVE_Z=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_Z',0x8519)
GL_TRANSPOSE_COLOR_MATRIX=_C('GL_TRANSPOSE_COLOR_MATRIX',0x84E6)
GL_TRANSPOSE_MODELVIEW_MATRIX=_C('GL_TRANSPOSE_MODELVIEW_MATRIX',0x84E3)
GL_TRANSPOSE_PROJECTION_MATRIX=_C('GL_TRANSPOSE_PROJECTION_MATRIX',0x84E4)
GL_TRANSPOSE_TEXTURE_MATRIX=_C('GL_TRANSPOSE_TEXTURE_MATRIX',0x84E5)
@_f
@_p.types(None,_cs.GLenum)
def glActiveTexture(texture):pass
@_f
@_p.types(None,_cs.GLenum)
def glClientActiveTexture(texture):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage1D(target,level,internalformat,width,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage2D(target,level,internalformat,width,height,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage3D(target,level,internalformat,width,height,depth,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage1D(target,level,xoffset,width,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage2D(target,level,xoffset,yoffset,width,height,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage3D(target,level,xoffset,yoffset,zoffset,width,height,depth,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,ctypes.c_void_p)
def glGetCompressedTexImage(target,level,img):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glLoadTransposeMatrixd(m):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glLoadTransposeMatrixf(m):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glMultTransposeMatrixd(m):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glMultTransposeMatrixf(m):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble)
def glMultiTexCoord1d(target,s):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMultiTexCoord1dv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat)
def glMultiTexCoord1f(target,s):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexCoord1fv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glMultiTexCoord1i(target,s):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glMultiTexCoord1iv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLshort)
def glMultiTexCoord1s(target,s):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLshortArray)
def glMultiTexCoord1sv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble)
def glMultiTexCoord2d(target,s,t):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMultiTexCoord2dv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat)
def glMultiTexCoord2f(target,s,t):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexCoord2fv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint)
def glMultiTexCoord2i(target,s,t):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glMultiTexCoord2iv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLshort,_cs.GLshort)
def glMultiTexCoord2s(target,s,t):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLshortArray)
def glMultiTexCoord2sv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMultiTexCoord3d(target,s,t,r):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMultiTexCoord3dv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glMultiTexCoord3f(target,s,t,r):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexCoord3fv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint)
def glMultiTexCoord3i(target,s,t,r):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glMultiTexCoord3iv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glMultiTexCoord3s(target,s,t,r):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLshortArray)
def glMultiTexCoord3sv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMultiTexCoord4d(target,s,t,r,q):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMultiTexCoord4dv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glMultiTexCoord4f(target,s,t,r,q):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexCoord4fv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glMultiTexCoord4i(target,s,t,r,q):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glMultiTexCoord4iv(target,v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLshort,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glMultiTexCoord4s(target,s,t,r,q):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLshortArray)
def glMultiTexCoord4sv(target,v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLboolean)
def glSampleCoverage(value,invert):pass



'''OpenGL extension SGIX.video_source

This module customises the behaviour of the 
OpenGL.raw.GLX.SGIX.video_source to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides the capability to source pixel data from a
	video stream.  A new type of GLXDrawable (GLXVideoSourceSGIX) is
	introduced which represents the drain node of a Video Library (VL)
	path.  A GLXVideoSourceSGIX may be specified as the <read> parameter
	to glXMakeCurrentReadSGI to indicate that pixel data should be read
	from the specified video source instead of from the framebuffer.  A
	GLXVideoSourceSGIX may only be specified as a <read> drawable.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/video_source.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.SGIX.video_source import *
from OpenGL.raw.GLX.SGIX.video_source import _EXTENSION_NAME

def glInitVideoSourceSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_texture_cube_map_array'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_texture_cube_map_array',error_checker=_errors._error_checker)
GL_INT_SAMPLER_CUBE_MAP_ARRAY_ARB=_C('GL_INT_SAMPLER_CUBE_MAP_ARRAY_ARB',0x900E)
GL_PROXY_TEXTURE_CUBE_MAP_ARRAY_ARB=_C('GL_PROXY_TEXTURE_CUBE_MAP_ARRAY_ARB',0x900B)
GL_SAMPLER_CUBE_MAP_ARRAY_ARB=_C('GL_SAMPLER_CUBE_MAP_ARRAY_ARB',0x900C)
GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_ARB=_C('GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_ARB',0x900D)
GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB=_C('GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB',0x900A)
GL_TEXTURE_CUBE_MAP_ARRAY_ARB=_C('GL_TEXTURE_CUBE_MAP_ARRAY_ARB',0x9009)
GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_ARB=_C('GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_ARB',0x900F)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_framebuffer_no_attachments'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_framebuffer_no_attachments',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_DEFAULT_FIXED_SAMPLE_LOCATIONS=_C('GL_FRAMEBUFFER_DEFAULT_FIXED_SAMPLE_LOCATIONS',0x9314)
GL_FRAMEBUFFER_DEFAULT_HEIGHT=_C('GL_FRAMEBUFFER_DEFAULT_HEIGHT',0x9311)
GL_FRAMEBUFFER_DEFAULT_LAYERS=_C('GL_FRAMEBUFFER_DEFAULT_LAYERS',0x9312)
GL_FRAMEBUFFER_DEFAULT_SAMPLES=_C('GL_FRAMEBUFFER_DEFAULT_SAMPLES',0x9313)
GL_FRAMEBUFFER_DEFAULT_WIDTH=_C('GL_FRAMEBUFFER_DEFAULT_WIDTH',0x9310)
GL_MAX_FRAMEBUFFER_HEIGHT=_C('GL_MAX_FRAMEBUFFER_HEIGHT',0x9316)
GL_MAX_FRAMEBUFFER_LAYERS=_C('GL_MAX_FRAMEBUFFER_LAYERS',0x9317)
GL_MAX_FRAMEBUFFER_SAMPLES=_C('GL_MAX_FRAMEBUFFER_SAMPLES',0x9318)
GL_MAX_FRAMEBUFFER_WIDTH=_C('GL_MAX_FRAMEBUFFER_WIDTH',0x9315)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glFramebufferParameteri(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferParameteriv(target,pname,params):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_OES_vertex_array_object'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_OES_vertex_array_object',error_checker=_errors._error_checker)
GL_VERTEX_ARRAY_BINDING_OES=_C('GL_VERTEX_ARRAY_BINDING_OES',0x85B5)
@_f
@_p.types(None,_cs.GLuint)
def glBindVertexArrayOES(array):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteVertexArraysOES(n,arrays):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenVertexArraysOES(n,arrays):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsVertexArrayOES(array):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_vertex_attrib_64bit'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_vertex_attrib_64bit',error_checker=_errors._error_checker)
GL_DOUBLE_MAT2=_C('GL_DOUBLE_MAT2',0x8F46)
GL_DOUBLE_MAT2x3=_C('GL_DOUBLE_MAT2x3',0x8F49)
GL_DOUBLE_MAT2x4=_C('GL_DOUBLE_MAT2x4',0x8F4A)
GL_DOUBLE_MAT3=_C('GL_DOUBLE_MAT3',0x8F47)
GL_DOUBLE_MAT3x2=_C('GL_DOUBLE_MAT3x2',0x8F4B)
GL_DOUBLE_MAT3x4=_C('GL_DOUBLE_MAT3x4',0x8F4C)
GL_DOUBLE_MAT4=_C('GL_DOUBLE_MAT4',0x8F48)
GL_DOUBLE_MAT4x2=_C('GL_DOUBLE_MAT4x2',0x8F4D)
GL_DOUBLE_MAT4x3=_C('GL_DOUBLE_MAT4x3',0x8F4E)
GL_DOUBLE_VEC2=_C('GL_DOUBLE_VEC2',0x8FFC)
GL_DOUBLE_VEC3=_C('GL_DOUBLE_VEC3',0x8FFD)
GL_DOUBLE_VEC4=_C('GL_DOUBLE_VEC4',0x8FFE)
GL_RGB32I=_C('GL_RGB32I',0x8D83)
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLdoubleArray)
def glGetVertexAttribLdv(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble)
def glVertexAttribL1d(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL1dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL2d(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL2dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL3d(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL3dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL4d(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL4dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glVertexAttribLPointer(index,size,type,stride,pointer):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_EXT_multisampled_render_to_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_EXT_multisampled_render_to_texture',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SAMPLES_EXT=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SAMPLES_EXT',0x8D6C)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT=_C('GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT',0x8D56)
GL_MAX_SAMPLES_EXT=_C('GL_MAX_SAMPLES_EXT',0x8D57)
GL_RENDERBUFFER_SAMPLES_EXT=_C('GL_RENDERBUFFER_SAMPLES_EXT',0x8CAB)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLsizei)
def glFramebufferTexture2DMultisampleEXT(target,attachment,textarget,texture,level,samples):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorageMultisampleEXT(target,samples,internalformat,width,height):pass



'''OpenGL extension NV.shader_atomic_float

This module customises the behaviour of the 
OpenGL.raw.GL.NV.shader_atomic_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides GLSL built-in functions and assembly opcodes
	allowing shaders to perform atomic read-modify-write operations to buffer
	or texture memory with floating-point components.  The set of atomic
	operations provided by this extension is limited to adds and exchanges.
	Providing atomic add support allows shaders to atomically accumulate the
	sum of floating-point values into buffer or texture memory across multiple
	(possibly concurrent) shader invocations.
	
	This extension provides GLSL support for atomics targeting image uniforms
	(if GLSL 4.20, ARB_shader_image_load_store, or EXT_shader_image_load_store
	is supported) or floating-point pointers (if NV_gpu_shader5 is supported).
	Additionally, assembly opcodes for these operations is also provided if
	NV_gpu_program5 is supported.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_atomic_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.shader_atomic_float import *
from OpenGL.raw.GL.NV.shader_atomic_float import _EXTENSION_NAME

def glInitShaderAtomicFloatNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.misc_attribute

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.misc_attribute to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_misc_attribute extends the list of attribute groups. It provides
	a miscellaneous group, controlled by the MISC_BIT_EXT bit, that contains
	the attribute state of extensions that don't logically fit in any other
	group. 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/misc_attribute.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.misc_attribute import *
from OpenGL.raw.GL.EXT.misc_attribute import _EXTENSION_NAME

def glInitMiscAttributeEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.clip_volume_hint

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.clip_volume_hint to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_clip_volume_hint provides a mechanism for applications to
	indicate that they do not require clip volume clipping for
	primitives. It allows applications to maximize performance in
	situations where they know that clipping is unnecessary.
	EXT_clip_volume_hint is only an indication, though, and
	implementations are free to ignore it.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/clip_volume_hint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.clip_volume_hint import *
from OpenGL.raw.GL.EXT.clip_volume_hint import _EXTENSION_NAME

def glInitClipVolumeHintEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.blend_equation_advanced

This module customises the behaviour of the 
OpenGL.raw.GL.NV.blend_equation_advanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a number of "advanced" blending equations that can be
	used to perform new color blending operations, many of which are more
	complex than the standard blend modes provided by unextended OpenGL.  This
	extension provides two different extension string entries:
	
	- NV_blend_equation_advanced:  Provides the new blending equations, but
	  guarantees defined results only if each sample is touched no more than
	  once in any single rendering pass.  The command BlendBarrierNV() is
	  provided to indicate a boundary between passes.
	
	- NV_blend_equation_advanced_coherent:  Provides the new blending
	  equations, and guarantees that blending is done coherently and in API
	  primitive ordering.  An enable is provided to allow implementations to
	  opt out of fully coherent blending and instead behave as though only
	  NV_blend_equation_advanced were supported.
	
	Some implementations may support NV_blend_equation_advanced without
	supporting NV_blend_equation_advanced_coherent.
	
	In unextended OpenGL, the set of blending equations is limited, and can be
	expressed very simply.  The MIN and MAX blend equations simply compute
	component-wise minimums or maximums of source and destination color
	components.  The FUNC_ADD, FUNC_SUBTRACT, and FUNC_REVERSE_SUBTRACT
	multiply the source and destination colors by source and destination
	factors and either add the two products together or subtract one from the
	other.  This limited set of operations supports many common blending
	operations but precludes the use of more sophisticated transparency and
	blending operations commonly available in many dedicated imaging APIs.
	
	This extension provides a number of new "advanced" blending equations.
	Unlike traditional blending operations using the FUNC_ADD equation, these
	blending equations do not use source and destination factors specified by
	BlendFunc.  Instead, each blend equation specifies a complete equation
	based on the source and destination colors.  These new blend equations are
	used for both RGB and alpha components; they may not be used to perform
	separate RGB and alpha blending (via functions like
	BlendEquationSeparate).
	
	These blending operations are performed using premultiplied colors, where
	RGB colors stored in the framebuffer are considered to be multiplied by
	alpha (coverage).  The fragment color may be considered premultiplied or
	non-premultiplied, according the BLEND_PREMULTIPLIED_SRC_NV blending
	parameter (as specified by the new BlendParameteriNV function).  If
	fragment color is considered non-premultiplied, the (R,G,B) color
	components are multiplied by the alpha component prior to blending.  For
	non-premultiplied color components in the range [0,1], the corresponding
	premultiplied color component would have values in the range [0*A,1*A].
	
	Many of these advanced blending equations are formulated where the result
	of blending source and destination colors with partial coverage have three
	separate contributions:  from the portions covered by both the source and
	the destination, from the portion covered only by the source, and from the
	portion covered only by the destination.  The blend parameter
	BLEND_OVERLAP_NV can be used to specify a correlation between source and
	destination pixel coverage.  If set to CONJOINT_NV, the source and
	destination are considered to have maximal overlap, as would be the case
	if drawing two objects on top of each other.  If set to DISJOINT_NV, the
	source and destination are considered to have minimal overlap, as would be
	the case when rendering a complex polygon tessellated into individual
	non-intersecting triangles.  If set to UNCORRELATED_NV (default), the
	source and destination coverage are assumed to have no spatial correlation
	within the pixel.
	
	In addition to the coherency issues on implementations not supporting
	NV_blend_equation_advanced_coherent, this extension has several
	limitations worth noting.  First, the new blend equations are not
	supported while rendering to more than one color buffer at once; an
	INVALID_OPERATION will be generated if an application attempts to render
	any primitives in this unsupported configuration.  Additionally, blending
	precision may be limited to 16-bit floating-point, which could result in a
	loss of precision and dynamic range for framebuffer formats with 32-bit
	floating-point components, and in a loss of precision for formats with 12-
	and 16-bit signed or unsigned normalized integer components.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/blend_equation_advanced.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.blend_equation_advanced import *
from OpenGL.raw.GL.NV.blend_equation_advanced import _EXTENSION_NAME

def glInitBlendEquationAdvancedNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.shading_language_100

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shading_language_100 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension string indicates that the OpenGL Shading Language is
	supported. The Shading Language is defined by a separate specification
	document which can be downloaded from
	
	    http://www.opengl.org/documentation/oglsl.html

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shading_language_100.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shading_language_100 import *
from OpenGL.raw.GL.ARB.shading_language_100 import _EXTENSION_NAME

def glInitShadingLanguage100ARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION